from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db.models.fields.files import FieldFile, FileField
from django.utils import six

from south.modelsinspector import introspector

from .storage import WeedFSStorage


class WeedFSFieldFile(FieldFile):

    def _split_name(self):
        splitted_name = self.name.split(':', 1)
        if len(splitted_name) == 2:
            return splitted_name
        return splitted_name[0], ''
            
    def _get_storage_fid(self):
        return self._split_name()[0]
    storage_fid = property(_get_storage_fid)

    def _get_verbose_name(self):
        return self._split_name()[1]
    verbose_name = property(_get_verbose_name)
   
    def _get_content(self):
        self._require_file()
        return self.storage.content(self.storage_fid)
    content = property(_get_content)

    def _get_url(self):
        self._require_file()
        content_type = ContentType.objects.get_for_model(self.instance._meta.model)
        return reverse('weedfs_get_file', kwargs={
            'content_type_id': content_type.id,
            'object_id': self.instance.id,
            'field_name': self.field.name,
            'file_name': self.verbose_name,
        })
    url = property(_get_url)

    def _get_storage_url(self):
        self._require_file()
        return self.storage.url(self.storage_fid)
    storage_url = property(_get_storage_url)
 

class WeedFSFileField(FileField):

    # The class to wrap instance attributes in. Accessing the file object off
    # the instance will always return an instance of attr_class.
    attr_class = WeedFSFieldFile

    def __init__(self, verbose_name=None, name=None, storage=None, **kwargs):
        if storage is None:
            storage = WeedFSStorage()

        super(WeedFSFileField, self).__init__(verbose_name, name,
            storage=storage, upload_to='/', **kwargs)

    def get_prep_value(self, value):
        "Returns field's value prepared for saving into a database."
        # Need to convert File objects provided via a form to unicode for database insertion
        if value is None:
            return None
        if isinstance(value, six.string_types):
            return six.text_type(value)
        if value.name == '':
            return ''
        if isinstance(value, WeedFSFieldFile):
            return value.name
        return self.storage.save(None, value)

    def south_field_triple(self):
        field_class = "django.db.models.fields.CharField"
        args, kwargs = introspector(self)
        return (field_class, args, kwargs)



from datetime import datetime
import os

from django.conf import settings
from django.core.files import File
from django.core.files.storage import Storage

from pyweed import WeedFS


DATE_IS_NOT_AVAILABLE = datetime.min


class WeedFSStorage(Storage):
    """
    Weed-FS storage.
    Weed-FS is a simple and highly scalable distributed file system.
    """

    def __init__(self, master_host=None, master_port=None):
        if master_host is None:
            master_host = settings.WEEDFS_MASTER_HOST
        self.master_host = master_host
        if master_port is None:
            master_port = settings.WEEDFS_MASTER_PORT
        self.master_port = master_port
        self.fs = WeedFS(master_host, master_port)

    def get_available_name(self, name):
        return os.path.basename(name)

    def content(self, name):
        return self.fs.get_file(name)

    def _save(self, name, content):
        fid = self.fs.upload_file(stream=content.file, name=name)
        content.close()
        if hasattr(content, 'temporary_file_path'):
            os.remove(content.temporary_file_path())
        return '%s:%s' % (fid, name)

    def delete(self, name):
        assert name, "The name argument is not allowed to be empty."
        self.fs.delete_file(name)

    def exists(self, name):
        return self.fs.file_exists(name)

    def size(self, name):
        return self.fs.get_file_size(name) or 0

    def url(self, name):
        return self.fs.get_file_url(name)

    def accessed_time(self, name):
        return DATE_IS_NOT_AVAILABLE

    def created_time(self, name):
        return DATE_IS_NOT_AVAILABLE

    def modified_time(self, name):
        return DATE_IS_NOT_AVAILABLE



from mimetypes import guess_type

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404, HttpResponse


def get_file(request, content_type_id, object_id, field_name, file_name):
    try:
        ct = ContentType.objects.get_for_id(content_type_id)
        obj = ct.get_object_for_this_type(id=object_id)
        field = getattr(obj, field_name)
    except (ObjectDoesNotExist, AttributeError):
        raise Http404
    if settings.ALLOW_NGINX_X_ACCEL_REDIRECT:
        response = HttpResponse()
        response['X-Accel-Redirect'] = field.storage_url
    else:
        response = HttpResponse(field.content,
            content_type=guess_type(field.verbose_name)[0])
    return response



from django.conf.urls import patterns, url

urlpatterns = patterns('djweed.views',
    url('^(?P<content_type_id>\d+)/(?P<object_id>\d+)/(?P<field_name>[\w\_]+)/(?P<file_name>[\w\.\_\-]*)$', 'get_file', name='weedfs_get_file'),
) 



#!/usr/bin/python

import os
from setuptools import setup, find_packages

from djweed.version import __version__


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ''


META_DATA = {
    'name': 'django-weed',
    'version': __version__,
    'description': read('DESCRIPTION'),
    'long_description': read('README.md'),
    'license': 'MIT',

    'author': "Vlad Frolov",
    'author_email': "frolvlad@gmail.com",

    'url': "https://github.com/ProstoKSI/django-weed",

    'packages': find_packages(),

    'install_requires': ('django', 'pyweed>=0.3.2', ),
}

if __name__ == "__main__":
    setup(**META_DATA)

